"""
作者：顾平安
联系方式：g332920287@foxmail.com

仅供技术研究使用，请勿商用或者非法用途，如有影响贵司或组织的权益，请联系我删除
"""
from .cli import execute
from .service import create_book, get_content

__all__ = ['execute', 'create_book', 'get_content']





import json
import re
import requests
from urllib.parse import unquote
from pathlib import Path
from . import const


class Book(object):
    def __init__(self, id_, title, author, book_type, doc_type, slug, desc):
        self.id = id_
        self.title = title
        self.author = author
        self.book_type = book_type
        self.doc_type = doc_type
        self.slug = slug
        self.desc = desc

    def __str__(self):
        return f"""{'BEGIN'.center(66, '-')}
标题：{self.title}({self.id})
作者：{self.author}  文档类型：{self.doc_type}  slug：{self.slug}
描述：{self.desc}
{'END'.center(66, '-')}
"""


def set_options(cookies, proxies):
    if cookies:
        const.headers['Cookies'] = cookies

    if proxies:
        try:
            proxy_list = proxies.split(',')
            proxy_dict = {scheme: proxy for scheme, proxy in (p.split('=') for p in proxy_list)}
        except ValueError:
            print('代理参数格式不正确，参考："http=proxy1,https=proxy2"')
            return None
        const.proxies.update(proxy_dict)


def get_save_path(raw_path: str, book: Book):
    raw_path = Path(raw_path)
    if raw_path.is_dir():
        if not raw_path.exists():
            print(f'警告：保存的路径不存在（{str(raw_path.absolute().resolve())}），默认保存在当前文件夹')
            raw_path = Path('./')
        save_path = raw_path / f'{book.title}.md'
    else:
        save_folder = raw_path.parent.resolve()
        save_filename = raw_path.name
        if not save_folder.exists():
            print(f'警告：保存的路径不存在（{str(save_folder.absolute().resolve())}），默认保存在当前文件夹')
            save_path = Path('./') / save_filename
        else:
            save_path = raw_path
    return save_path


def create_book(url):
    page_html_resp = requests.request('GET', url, headers=const.headers, proxies=const.proxies)
    page_html = page_html_resp.text
    match_result = re.findall(r'decodeURIComponent\("(.*)"\)', page_html)
    raw_data = match_result[0] if match_result else None
    if raw_data:
        try:
            page_data = json.loads(unquote(raw_data))
            book = page_data['book']
            group = page_data['group']
            doc = page_data['doc']

            book_id = book['id']
            book_type = book['type']
            doc_title = doc['title']
            doc_type = doc['type']
            doc_desc = doc['description']
            doc_slug = doc['slug']
            book_author = group['name']

            return Book(book_id, doc_title, book_author, book_type, doc_type, doc_slug, doc_desc)
        except KeyError:
            return None


def get_content(book):
    api = f'https://www.yuque.com/api/docs/{book.slug}'
    if book.id:
        const.content_params['book_id'] = book.id
    response = requests.request('GET', api, params=const.content_params, proxies=const.proxies, headers=const.headers)
    result = response.json()
    try:
        content = result['data']['content']
        return content
    except KeyError:
        return None



import click
from click.testing import CliRunner
import lakedoc
from lakedoc import string
from . import const, service


@click.command()
@click.argument('url', default='')
@click.option('--path', 'path', '-p', default='./', help='指定 md 文档保存的路径')
@click.option('--savesource', 'savesource', '-s', is_flag=True, help='是否保存源 HTML 内容')
@click.option('--cookies', 'cookies', '-c', default='', help='携带 cookies 访问')
@click.option('--proxies', 'proxies', '-p', default='', help='指定代理："http=proxy1,https=proxy2"')
@click.option('--encoding', 'encoding', '-e', default='utf-8', help='指定保存文件的编码')
def cli_execute(url, path, savesource, cookies, proxies, encoding):
    if not url or not const.url_pattern.match(url):
        print(string.color_string('Error：语雀文档地址不正确，参考：https://www.yuque.com/.../.../...', 'red'))
        return None

    service.set_options(cookies, proxies)

    book = service.create_book(url)
    if not book:
        print(string.color_string('Error：获取知识库文档对象失败，可能文档已被移动、删除或不被支持', 'red'))
        return None
    print(str(book))

    content = service.get_content(book)
    if savesource:
        with open(f'./{book.title}.html', 'w', encoding=encoding) as fw:
            fw.write(content)

    save_path = service.get_save_path(path, book)
    lakedoc.convert(content, save_path, encoding=encoding, is_file=False, builder='lxml', title=f'# {book.title}')
    print(string.color_string(f'> 文档已保存，路径为：{str(save_path.absolute().resolve())}\n', 'green'))


def execute(url: str, path='./', savesource=False, cookies='', proxies='', encoding='utf-8', nohint=False):
    """
    执行主要功能的 Python 方法

    :param url: 要访问的 URL
    :param path: 保存路径
    :param savesource: 是否保存源 HTML 内容
    :param cookies: 携带的 cookies
    :param proxies: 指定的代理
    :param encoding: 保存文件的编码
    :param nohint: 是否打印提示
    """
    args = [url]
    if path:
        args.extend(['--path', path])
    if savesource:
        args.append('--savesource')
    if cookies:
        args.extend(['--cookies', cookies])
    if proxies:
        args.extend(['--proxies', proxies])
    if encoding:
        args.extend(['--encoding', encoding])

    runner = CliRunner()
    result = runner.invoke(cli_execute, args)

    if not nohint:
        print(result.output)



import re

url_pattern = re.compile(r'^https://www\.yuque\.com/([a-zA-Z0-9_-]+)/([a-zA-Z0-9_-]+)/([a-zA-Z0-9_-]+)$')

proxies = {'http': None, 'https': None}
headers = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
    'Accept-Encoding': 'gzip, deflate, br, zstd',
    'Accept-Language': 'zh-CN,zh;q=0.9',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36',
    'Host': 'www.yuque.com'
}

content_params = {
    'include_contributors': True,
    'include_like': True,
    'include_hits': True,
    'merge_dynamic_data': False,
}